//
// Camera.cpp
//

// Game includes.
#include "Camera.h"

// Engine includes.
#include "Box.h"
#include "DisplayManager.h"
#include "EventMouse.h"
#include "EventStep.h"
#include "LogManager.h"	
#include "WorldManager.h"

Camera::Camera() {

  // Camera attributes.
  setType("Camera");

  // View follows Camera, starting in middle of world.
  df::Box b = WM.getBoundary();
  setPosition(df::Vector(b.getHorizontal()/2.0f, b.getVertical()/2.0f));
  WM.setViewFollowing(this);

  // Camera, as ViewObject, lives in screen.  Start in center.
  b = WM.getView();
  setPosition(df::Vector(b.getHorizontal()/2.0f, b.getVertical()/2.0f));

  // Mouse controls Camera position.
  registerInterest(df::MSE_EVENT);

  // Move view if Camera at edge of screen.
  registerInterest(df::STEP_EVENT);
}

// Handle event.
int Camera::eventHandler(const df::Event *p_e) {

  // Mouse moves Camera around world.
  if (p_e->getType() == df::MSE_EVENT) {
    const df::EventMouse *p_me = static_cast <const df::EventMouse *> (p_e);
    if (p_me -> getMouseAction() == df::MOVED) {
      df::Vector new_pos = p_me->getMousePosition();
      setPosition(new_pos);
      return true;
    }
  }

  // Call WM.moveObject() to control Camera, then move back.
  if (p_e->getType() == df::STEP_EVENT) {
    df::Vector pos = getPosition(); 
    WM.moveObject(this, df::viewToWorld(pos));
    setPosition(pos); // Move back.
    return true;
  }

  return false;
}

// Draw Camera cursor on screen.
int Camera::draw() {
  return DM.drawCh(df::viewToWorld(getPosition()), CURSOR_CHAR, CURSOR_COLOR); 
}
